#pragma vertex
#pragma include "zframework/modules/render/shaders/glsl/libs/Default.vert.glsl"

void main() {
    defaultMain();
}

#pragma fragment
#pragma include "zframework/modules/render/shaders/glsl/libs/Default.frag.glsl"

uniform vec4 u_fillcolor;

#if defined(MONOCHROME_SPRITE_COLOR_BLENDING)
    const vec4 kRGBToYPrime = vec4(0.299, 0.587, 0.114, 0.0);
    const vec4 kRGBToI = vec4(0.596, -0.275, -0.321, 0.0);
    const vec4 kRGBToQ = vec4(0.212, -0.523, 0.311, 0.0);

    const vec4 kYIQToR = vec4(1.0, 0.956, 0.621, 0.0);
    const vec4 kYIQToG = vec4(1.0, -0.272, -0.647, 0.0);
    const vec4 kYIQToB = vec4(1.0, -1.107, 1.704, 0.0);
#endif

void main() {
    vec4 color = getColor();
    vec4 fillColor = u_fillcolor;

    #if defined(MONOCHROME_SPRITE_ALPHA_PREMULTIPLIED)
        // premultiply fill color if the texture also has alpha premultiplied (using texture alpha value)
        fillColor = vec4(fillColor.rgb * color.a, fillColor.a);
    #endif

    #if defined(MONOCHROME_SPRITE_COLOR_BLENDING)
        // From original color
        float YPrime = dot(color, kRGBToYPrime);

        // From provided color
        float I = dot(fillColor, kRGBToI);
        float Q = dot(fillColor, kRGBToQ);

        // Convert back to RGB
        vec4 yIQ = vec4(YPrime, I, Q, 0.0);
        color.r = dot(yIQ, kYIQToR);
        color.g = dot(yIQ, kYIQToG);
        color.b = dot(yIQ, kYIQToB);
        color.a = color.a * fillColor.a;
    #elif defined(MONOCHROME_SPRITE_HUE_BLENDING)
        vec3 hsvColor = rgb2hsv(color.rgb);

        // Replace hue color component
        hsvColor.x = rgb2hsv(fillColor.rgb).x;

        color = vec4(hsv2rgb(hsvColor), color.a * fillColor.a);
    #else
        color = grayscale(color);
    #endif

    gl_FragColor = color;
    alphaTest();
}
